@@ -1,13 +1,19 @@
 import logging
+import os
 import sys
+import time
 from collections import defaultdict
-from typing import Optional
+from pathlib import Path
+from typing import Optional, Tuple
 
 import click
 from click.core import Command, Context
+from dbt.main import main as dbt_main
 from dotenv import load_dotenv
 
 from panoramic.cli.__version__ import __version__
+from panoramic.cli.analytics import is_enabled, write_command_event
+from panoramic.cli.command import configure_anonymous_analytics
 from panoramic.cli.errors import (
     CompanyNotFoundException,
     SourceNotFoundException,
@@ -18,20 +24,51 @@ from panoramic.cli.errors import (
 )
 from panoramic.cli.paths import Paths
 from panoramic.cli.print import echo_error, echo_errors, echo_info, echo_warnings
+from panoramic.cli.validate import validate_dbt
+
+_PROFILES_DIR_ARG = '--profiles-dir'
+_PROJECT_DIR_ARG = '--project-dir'
 
 
-class ConfigAwareCommand(Command):
+class CommandWithAnalytics(Command):
+    """Execute command and write usage metrics to a file that is conditionally flushed to external system."""
+
+    def invoke(self, ctx: Context):
+        # This will make sure to display opt-out message for existing users that already run pano configure.
+        if not is_enabled():
+            configure_anonymous_analytics()
+
+        # If command is a subcommand, combine its name with the command name.
+        group = ''
+        if ctx.parent is not None and ctx.parent.command != cli:  # type: ignore
+            group = ctx.parent.command.name  # type: ignore
+
+        start_time = time.time()
+        try:
+            result = super().invoke(ctx)
+            write_command_event(self.name, group, start_time)
+            return result
+        except Exception as e:
+            write_command_event(self.name, group, start_time, error=str(e))
+            raise e
+
+
+class ConfigAwareCommand(CommandWithAnalytics):
     """Perform config file validation before running command."""
 
     def invoke(self, ctx: Context):
         from panoramic.cli.validate import validate_config
 
+        cwd = Path.cwd()
+
         try:
             validate_config()
             return super().invoke(ctx)
         except ValidationError as e:
             echo_error(str(e))
             sys.exit(1)
+        finally:
+            os.chdir(cwd)  # DBT sets cwd so we need to reset it
 
 
 class ConnectionAwareCommand(ConfigAwareCommand):
@@ -66,6 +103,22 @@ class ContextAwareCommand(ConfigAwareCommand):
             sys.exit(1)
 
 
+class DbtCommand(ContextAwareCommand):
+    """Command proxied to DBT CLI tool."""
+
+    def invoke(self, ctx: Context):
+        from panoramic.cli.dbt import prepare_dbt_project
+
+        try:
+            validate_dbt()
+            prepare_dbt_project()
+            return super().invoke(ctx)
+        except Exception as e:
+            # TODO: Catch DBT exception here?
+            echo_error(str(e))
+            sys.exit(1)
+
+
 class LocalStateAwareCommand(ContextAwareCommand):
     """Perform config, context, and local state files validation before running command."""
 
@@ -172,7 +225,7 @@ def list_companies():
     list_companies_command()
 
 
-@cli.command(help='Validate local files')
+@cli.command(help='Validate local files', cls=CommandWithAnalytics)
 @handle_exception
 def validate():
     from panoramic.cli.command import validate as validate_command
@@ -405,3 +458,69 @@ def test(name: str):
     from panoramic.cli.connections import test_connections_command
 
     test_connections_command(name)
+
+
+@cli.group()
+def analytics():
+    """Analytics subcommand for managing anonymous usage metrics collection preferences."""
+    pass
+
+
+@analytics.command(name='on', cls=ConfigAwareCommand)
+def analytics_on():
+    """Opt in to anonymous usage analytics.
+
+    pano analytics on
+    """
+    from panoramic.cli.analytics import opt_in_command
+
+    opt_in_command()
+
+
+@analytics.command(name='off', cls=ConfigAwareCommand)
+def analytics_off():
+    """Opt out off anonymous usage analytics.
+
+    pano analytics off
+    """
+    from panoramic.cli.analytics import opt_out_command
+
+    opt_out_command()
+
+
+@analytics.command(name='id', cls=ConfigAwareCommand)
+def analytics_id():
+    """Display current anonymous usage tracking id.
+
+    pano analytics id
+    """
+    from panoramic.cli.analytics import show_tracking_id_command
+
+    show_tracking_id_command()
+
+
+@cli.command(context_settings=dict(ignore_unknown_options=True), cls=DbtCommand)
+@click.argument('dbt_args', nargs=-1, type=click.UNPROCESSED)
+def dbt(dbt_args: Tuple[str]):
+    """Run DBT command.
+
+    \b
+    Supported commands:
+      pano dbt deps     Update package dependencies from pano.yaml
+      pano dbt compile  Compile pre-model transforms into .dbt/target/compiled
+      pano dbt run      Run compiled transforms against target defined in pano.yaml
+    """
+    # Handle no args passed
+    if len(dbt_args) == 0:
+        ctx = click.get_current_context()
+        click.echo(ctx.get_help())
+        return
+
+    args = list(dbt_args)
+
+    if _PROFILES_DIR_ARG not in args:
+        args.extend([_PROFILES_DIR_ARG, str(Paths.dbt_config_dir())])
+    if _PROJECT_DIR_ARG not in args:
+        args.extend([_PROJECT_DIR_ARG, str(Paths.dbt_project_dir())])
+
+    return dbt_main(args=args)
